//ASH-B - AkkuSpurwechselHindi mit Barcode-Erkennung
//Hindernisfahrzeug mit
//- Spurwechsel per Zufallsgenerator
//- Kollisionserkennung
//- Einstellung minSpeed und maxSpeed über Poti
//- Einstellung Frequenz Lenkvorgänge über Poti
//- Blaulicht
//- Programmierung über auf die Schiene aufgeklebte Barcodes

//Konfiguration
bool usePotSpeed = true; //Ist ein Poti Geschwindigkeit angeschlossen (sonst wird maxSpeed als Standardwert verwendet)?
bool usePotLaneChange = true; //Ist ein Poti für Frequenz Lenkvorgänge angeschlossen (sonst wird laneChangeMin und -Max verwendet)?
bool useInfrared = false; //ASH-B-Funktion verwenden? Auf false ändern, wenn kein Infrarotmodul angeschlossen ist
byte minSpeed = 155; //Start-Geschwindigkeit (0-255)
byte maxSpeed = 255; //Maximalgeschwindigkeit (0-255)
const byte afterCollisionSpeed = 200; //Anfahr-Geschwindigkeit nach Kollision (0-255)
const int ledIntervalTurn = 300; //Blink-Intervall in Millisekunden
const int ledIntervalHazard = 100; //Blink-Intervall für Warnblinkanlage
const int ledIntervalFlash = 350; //Blink-Intervall für Blaulicht
const int driveInterval = 12; //Beschleunigungsintervall in Millisekunden (kleiner = schneller)
const int collisionWait = 1500; //Wartedauer nach Kollision, bevor der Hindi wieder anfährt
const int signalBeforeWait = 1000; //Zeitraum in Millisekunden, in dem vor einem Spurwechsel geblinkt wird
const int signalAfterWait = 1500; //Blinker bleibt bei Spurwechsel so lange laufen
long laneChangeMin = 1500; //Mindest-Zeitraum in Millisekunden zwischen Spurwechseln
long laneChangeMax = 5000; //Höchst-Zeitraum in Millisekunden zwischen Spurwechseln
const int bounceTimePress = 5; //Zeit in Millisekunden, in der der Taster mindestens ausgelöst sein muss, um eine Kollision zu werten
const int bounceTimeRelease = 200; //Prellzeit für Taster nach Lösen
//Konfiguration Barcode-Erkennung
const int breakIrReadAfter = 100; //Intervall, nach dem mit dem Lesen von IR-Impulsen aufgehört wird, falls kein neuer Strich mehr kommt
const int minIrImpulseInterval = 5; //5 15; //Mindestleselänge von Barcode-Streifen in Millisekunden
const int delayLaneChangeDuration = 1000; //Nicht lenken-Code: Zeitraum in Millisekunden, in dem nicht gelenkt wird
const int idleDuration = 300; //Leerlauf: Zeitraum in Millisekunden

//Motorsteuerung
bool wheelState = false; //false ist rechts, true ist links - wird zum vorher setzen der Blinker verwendet
byte currentSpeed = minSpeed;
bool currentWheelState = false; //tatsächlich aktueller Lenkstatus

//Ablaufsteuerung
//Kollision
bool modeCollision = false;
bool modeWait = false;
unsigned long previousCollisionMillis = 0;
unsigned long switchMillis = 0;
unsigned long previousCollisionPressMillis = 0;
//LEDs
bool turnLeft = false; //Blinker links an
bool ledLeftOn = false;
bool turnRight = false; //Blinker rechts an
bool ledRightOn = false;
bool ledFlashOn = false; //Blaulicht an
bool hazard = false; //Warnblinkanlage
//bool ledsOff = false;
bool ledStateChange = false;
unsigned long previousLedMillis = 0;
unsigned long previousLedMillisFlash = 0;
//Fahren
unsigned long previousDriveMillis = 0;
//Spurwechsel
bool changePlanned = false;
bool modeChange = false;
bool beforeChange = false;
long laneChangeTime = 0; //Timestamp, wann der nächste Spurwechsel passieren soll
unsigned long changeMillis = 0;
unsigned long previousChangeMillis = 0;
//Leerlauf
bool idleOn = false;
unsigned long previousIdleMillis = 0;
//Turbofunktion
bool turboOn = false;
bool ledTurboOn = false;
int turboDuration = 0;
unsigned long previousTurboMillis = 0;
unsigned long previousTurboLedMillis = 0;
//Barcode
unsigned long start;
bool impulseReading = false;
bool currentlyReading = false;
int impulseCounter = 0;
bool startBlack = false;
bool black = false;
bool startWhite = false;
bool codeFound = false;
int turboTime = 0;

#define pinInSpeed A0 //Poti für Einstellung min/maxSpeed
#define pinInChangeFrq A1 //Poti für Einstellung Anzahl Lenkvorgänge
#define pinInCollision 10 //Mikroschalter für Kollisionserkennung
#define pinInIr 9 //Eingang für Infrarot-Modul
#define pinInRandom A6 //Nicht verbundener Pin für randomSeed

#define pinOutMotorLeft 7 //PWM-Output für Motor
#define pinOutMotorRight 8 //PWM-Output für Motor
#define pinOutLedTurnLeft 4 //Blinker links
#define pinOutLedTurnRight 5 //Blinker rechts
#define pinOutLedFlashLeft 2 //Blaulicht
#define pinOutLedFlashRight 3 //Blaulicht
//#define pinOutLedStatus 3

void setup() {
  pinMode(pinInCollision, INPUT);
  pinMode(pinInIr, INPUT);
  
  pinMode(pinOutMotorLeft, OUTPUT); //PWM-Output für Motor
  pinMode(pinOutMotorRight, OUTPUT); //PWM-Output für Motor
  pinMode(pinOutLedTurnLeft, OUTPUT);
  pinMode(pinOutLedTurnRight, OUTPUT);
  pinMode(pinOutLedFlashLeft, OUTPUT);
  pinMode(pinOutLedFlashRight, OUTPUT);
  //pinMode(pinOutLedStatus, OUTPUT);

  randomSeed(analogRead(pinInRandom));

  //eine Sekunde warten, bis das Fahrzeug losfährt - Zeit mit LEDs flashen überbrücken ;)
  for(int i=0; i<3; i++) {
    digitalWrite(pinOutLedTurnLeft, HIGH);
    delay(100);
    digitalWrite(pinOutLedTurnLeft, LOW);
    digitalWrite(pinOutLedTurnRight, HIGH);
    delay(100);
    digitalWrite(pinOutLedTurnRight, LOW);
    digitalWrite(pinOutLedFlashLeft, HIGH);
    delay(100);
    digitalWrite(pinOutLedFlashLeft, LOW);
    digitalWrite(pinOutLedFlashRight, HIGH);
    delay(100);
    digitalWrite(pinOutLedFlashRight, LOW);
  }
}

void loop() {
  readPots();
  handleLeds();
  handleCollision();
  handleIr();
  drive();
  laneChange();
}
